package kr.co.modacom.iot.ltegwdev.model.type;

public enum RspCode {
	
	ACCEPTED	(	1000	,"	ACCEPTED "),
	OK	(	2000	,"	OK	"),
	CREATED	(	2001	,"	CREATED	"),
	DELETED	(	2002	,"	DELETED	"),
	UPDATED	(	2004	,"	UPDATED	"),
	
	BAD_REQUEST	(	4000	,"	BAD_REQUEST	"),
	NOT_FOUND	(	4004	,"	NOT_FOUND	"),
	OPERATION_NOT_ALLOWED	(	4005	,"	OPERATION_NOT_ALLOWED	"),
	REQUEST_TIMEOUT	(	4008	,"	REQUEST_TIMEOUT	"),
	SUBSCRIPTION_CREATOR_HAS_NO_PRIVILEGE	(	4101	,"	SUBSCRIPTION_CREATOR_HAS_NO_PRIVILEGE	"),
	CONTENTS_UNACCEPTABLE	(	4102	,"	CONTENTS_UNACCEPTABLE	"),
	ACCESS_DENIED	(	4103	,"	ACCESS_DENIED	"),
	ACCESS_DENIED_IR_LEARNING_IN_PROGRESS	(	4104 ,"ACCESS_DENIED_IR_LEARNING_IN_PROGRESS"),
	GROUP_REQUEST_IDENTIFIER_EXISTS	(	4105	,"	GROUP_REQUEST_IDENTIFIER_EXISTS	"),
	CONFLICT	(	4106	,"	CONFLICT	"),
	INTERNAL_SERVER_ERROR	(	5000	,"	INTERNAL_SERVER_ERROR	"),
	NOT_IMPLEMENTED	(	5001	,"	NOT_IMPLEMENTED	"),
	TARGET_NOT_REACHABLE	(	5103	,"	TARGET_NOT_REACHABLE	"),
	NO_PRIVILEGE	(	5105	,"	NO_PRIVILEGE	"),
	ALREADY_EXISTS	(	5106	,"	ALREADY_EXISTS	"),
	TARGET_NOT_SUBSCRIBABLE	(	5203	,"	TARGET_NOT_SUBSCRIBABLE	"),
	SUBSCRIPTION_VERIFICATION_INITIATION_FAILED	(	5204	,"	SUBSCRIPTION_VERIFICATION_INITIATION_FAILED	"),
	SUBSCRIPTION_HOST_HAS_NO_PRIVILEGE	(	5205	,"	SUBSCRIPTION_HOST_HAS_NO_PRIVILEGE	"),
	NON_BLOCKING_REQUEST_NOT_SUPPORTED	(	5206	,"	NON_BLOCKING_REQUEST_NOT_SUPPORTED	"),
	EXTENAL_OBJECT_NOT_REACHABLE	(	6003	,"	EXTENAL_OBJECT_NOT_REACHABLE	"),
	EXTENAL_OBJECT_NOT_FOUND	(	6005	,"	EXTENAL_OBJECT_NOT_FOUND	"),
	MAX_NUMBERF_OF_MEMBER_EXCEEDED	(	6010	,"	MAX_NUMBERF_OF_MEMBER_EXCEEDED	"),
	MEMBER_TYPE_INCONSISTENT	(	6011	,"	MEMBER_TYPE_INCONSISTENT	"),
	MGMT_SESSION_CANNOT_BE_ESTABLISHED	(	6020	,"	MGMT_SESSION_CANNOT_BE_ESTABLISHED	"),
	MGMT_SESSION_ESTABLISHMENT_TIMEOUT	(	6021	,"	MGMT_SESSION_ESTABLISHMENT_TIMEOUT	"),
	INVALID_CMDTYPE	(	6022	,"	INVALID_CMDTYPE	"),
	INVALID_ARGUMENTS	(	6023	,"	INVALID_ARGUMENTS	"),
	INSUFFICIENT_ARGUMENTS	(	6024	,"	INSUFFICIENT_ARGUMENTS	"),
	MGMT_CONVERSION_ERROR	(	6025	,"	MGMT_CONVERSION_ERROR	"),
	MGMT_CANCELATION_FAILURE	(	6026	,"	MGMT_CANCELATION_FAILURE	"),
	ALREADY_COMPLETE	(	6028	,"	ALREADY_COMPLETE	"),
	COMMAND_NOT_CANCELLABLE	(	6029	,"	COMMAND_NOT_CANCELLABLE	"),
	OMNIPUB_NOT_CONNECTED	(	6030, 	"OMNIPUB_NOT_CONNECTED"),
	GATEWAY_DISCONNECTED (	6050	,"	GATEWAY_DISCONNECTED	"),
	GATEWAY_NO_RESPONSE (	6060	,"	GATEWAY_NO_RESPONSE	"),
	INVALID_HOMEID	(	6070	, "	INVALID_HOMEID"),
	FUNCTIONAL_ERROR	(	7000	,"	FUNCTIONAL_ERROR"),
	PASSWORD_RULE_MISMATCH	(	7001	,"	PASSWORD_RULE_MISMATCH	"),
	PASSWORD_AUTH_ERROR	(	7002	,"	PASSWORD_AUTH_ERROR	"),
	GWID_REQUIRED	(	7003	,"	GWID_REQUIRED	"),
	GWID_NOT_INCLUDED	(	7004	,"	GWID_NOT_INCLUDED	"),
	GWID_MISMATCH	(	7005	,"	GWID_MISMATCH	"),
	USER_DATA_EXIST (7006, "USER_DATA_EXIST"),
	NOT_AUTHORIZED (7008, "NOT_AUTHORIZED"),
	USER_ALREADY_LOGIN (7009, "USER_ALREADY_LOGIN"),
	FAIL_AUTHORIZED_INFOMATION	(	7010	,"	FAIL_AUTHORIZED_INFOMATION	"),
	
	HUE_BRIDGE_NOT_RESPONDING	(	7700	,"	HUE_BRIDGE_NOT_RESPONDING	"),
	HUE_AUTHENTICATION_REQUIRED	(	7701	,"	HUE_AUTHENTICATION_REQUIRED	"),
	HUE_AUTHENTICATION_FAILED	(	7702	,"	HUE_AUTHENTICATION_FAILED	"),
	
	INSERT_FAILED		(	8500	,"	INSERT_FAILED	"),
	UPDATE_FAILED		(	8501	,"	UPDATE_FAILED	"),
	DELETE_FAILED		(	8502	,"	DELETE_FAILED	"),
	SELECT_FAILED		(	8503	,"	SELECT_FAILED	"),
	
	/* 서버 에러 */
	HTTP_UNAUTHORIZED 	(	90001, "HTTP_UNAUTHORIZED"),
	HTTP_REQUEST_TIMEOUT 	(	90002, "HTTP_REQUEST_TIMEOUT"),
	
	
	/* App 내부 처리 코드 */
	APP_SEND_FAIL_TO_SB			(91001, "APP_SEND_FAIL_TO_SB"),
	APP_SEND_FAIL_TO_SERVER		(91002, "APP_SEND_FAIL_TO_SERVER"),
	APP_TIMEOUT_FROM_SB			(91003, "APP_TIMEOUT_FROM_SB"),
	APP_TIMEOUT_FROM_SERVER		(91004, "APP_TIMEOUT_FROM_SERVER"),
	
	APP_PROCESS_REQUEST_FAIL	(91005, "APP_PROCESS_REQUEST_FAIL");
	
	
	private int code;
	private String name;
	
	private RspCode(int code, String name) {
		this.code = code;
		this.name = name;
	}
	
	public int getCode() {
		return code;
	}
	
	public String getName() {
		return name;
	}
	
	public static RspCode valueOf(int code) {
        switch (code) {
        case	1000	:	return	ACCEPTED	;
        case	2000	:	return	OK	;
        case	2001	:	return	CREATED	;
        case	2002	:	return	DELETED	;
        case	2004	:	return	UPDATED	;
        
        case	4000	:	return	BAD_REQUEST	;
        case	4004	:	return	NOT_FOUND	;
        case	4005	:	return	OPERATION_NOT_ALLOWED	;
        case	4008	:	return	REQUEST_TIMEOUT	;
        case	4101	:	return	SUBSCRIPTION_CREATOR_HAS_NO_PRIVILEGE	;
        case	4102	:	return	CONTENTS_UNACCEPTABLE	;
        case	4103	:	return	ACCESS_DENIED	;
        case	4104	:	return	ACCESS_DENIED_IR_LEARNING_IN_PROGRESS;
        case	4105	:	return	GROUP_REQUEST_IDENTIFIER_EXISTS	;
        case	4106	:	return	CONFLICT	;
        case	5000	:	return	INTERNAL_SERVER_ERROR	;
        case	5001	:	return	NOT_IMPLEMENTED	;
        case	5103	:	return	TARGET_NOT_REACHABLE	;
        case	5105	:	return	NO_PRIVILEGE	;
        case	5106	:	return	ALREADY_EXISTS	;
        case	5203	:	return	TARGET_NOT_SUBSCRIBABLE	;
        case	5204	:	return	SUBSCRIPTION_VERIFICATION_INITIATION_FAILED	;
        case	5205	:	return	SUBSCRIPTION_HOST_HAS_NO_PRIVILEGE	;
        case	5206	:	return	NON_BLOCKING_REQUEST_NOT_SUPPORTED	;
        case	6003	:	return	EXTENAL_OBJECT_NOT_REACHABLE	;
        case	6005	:	return	EXTENAL_OBJECT_NOT_FOUND	;
        case	6010	:	return	MAX_NUMBERF_OF_MEMBER_EXCEEDED	;
        case	6011	:	return	MEMBER_TYPE_INCONSISTENT	;
        case	6020	:	return	MGMT_SESSION_CANNOT_BE_ESTABLISHED	;
        case	6021	:	return	MGMT_SESSION_ESTABLISHMENT_TIMEOUT	;
        case	6022	:	return	INVALID_CMDTYPE	;
        case	6023	:	return	INVALID_ARGUMENTS	;
        case	6024	:	return	INSUFFICIENT_ARGUMENTS	;
        case	6025	:	return	MGMT_CONVERSION_ERROR	;
        case	6026	:	return	MGMT_CANCELATION_FAILURE	;
        case	6028	:	return	ALREADY_COMPLETE	;
        case	6029	:	return	COMMAND_NOT_CANCELLABLE	;
        case	6030	:	return	OMNIPUB_NOT_CONNECTED;
        case	6050	:	return 	GATEWAY_DISCONNECTED;
        case	6060	:	return 	GATEWAY_NO_RESPONSE;
        case	6070	:	return	INVALID_HOMEID;
        case	7000	:	return	FUNCTIONAL_ERROR;
        case	7001	:	return	PASSWORD_RULE_MISMATCH	;
        case	7002	:	return	PASSWORD_AUTH_ERROR	;
        case	7003	:	return	GWID_REQUIRED	;
        case	7004	:	return	GWID_NOT_INCLUDED	;
        case	7005	:	return	GWID_MISMATCH	;
        case 	7006	:	return 	USER_DATA_EXIST;
        case 	7008	: 	return  USER_ALREADY_LOGIN;
        case 	7700	:	return 	HUE_BRIDGE_NOT_RESPONDING;
        case 	7701	:	return 	HUE_AUTHENTICATION_REQUIRED;
        case 	7702	:	return 	HUE_AUTHENTICATION_FAILED;
        case 	8500	:	return 	INSERT_FAILED;
        case 	8501	:	return 	UPDATE_FAILED;
        case 	8502	:	return 	DELETE_FAILED;
        case 	8503	:	return 	SELECT_FAILED;
        
        case 	90001	:	return 	HTTP_UNAUTHORIZED;
        case 	90002	:	return 	HTTP_REQUEST_TIMEOUT;
        
        
        case 	91001	:	return 	APP_SEND_FAIL_TO_SB;
        case 	91002	:	return 	APP_SEND_FAIL_TO_SERVER;
        case 	91003	:	return 	APP_TIMEOUT_FROM_SB;
        case 	91004	:	return 	APP_TIMEOUT_FROM_SERVER;
        
        case 	91005	:	return 	APP_PROCESS_REQUEST_FAIL;

        }
        throw new IllegalArgumentException("Invalid ID");
	}

}
